diff -urN otp_src_R13B03.orig/erts/emulator/beam/erl_bif_re.c otp_src_R13B03.fix-re/erts/emulator/beam/erl_bif_re.c
--- otp_src_R13B03.orig/erts/emulator/beam/erl_bif_re.c	2009-03-12 21:15:38.000000000 +0900
+++ otp_src_R13B03.fix-re/erts/emulator/beam/erl_bif_re.c	2009-12-15 14:45:55.548029064 +0900
@@ -1119,6 +1119,32 @@
     restartp->extra.restart_data = &restartp->restart_data;
     restartp->extra.restart_flags = 0;
     
+    if( (restartp->flags & RESTART_FLAG_SUBJECT_IN_BINARY) && restartp->restart_data != NULL ) {
+	Eterm real_bin;
+	Uint offset;
+	Eterm* bptr;
+	int bitoffs;
+	int bitsize;
+	ProcBin* pb;
+	char* new_subject;
+
+	ASSERT( is_binary(BIF_ARG_1) );
+	ERTS_GET_REAL_BIN(BIF_ARG_1, real_bin, offset, bitoffs, bitsize);
+
+	bptr = binary_val(real_bin);
+	ASSERT( !(bitsize != 0 || bitoffs != 0 ||  (*bptr != HEADER_PROC_BIN)) );
+	pb = (ProcBin *) bptr;
+	new_subject = (char *) (pb->bytes+offset);
+	if( new_subject != restartp->subject ) {
+	    int slength;
+	    int distance;
+	    slength  = binary_size(BIF_ARG_1);
+	    distance = new_subject - restartp->subject;
+	    erts_pcre_restart_data_moved(restartp->restart_data, restartp->subject, slength, distance);
+	    restartp->subject = new_subject;
+	}
+    }
+
 #ifdef DEBUG
     loop_count = 0xFFFFFFFF;
 #endif
diff -urN otp_src_R13B03.orig/erts/emulator/pcre/pcre.h otp_src_R13B03.fix-re/erts/emulator/pcre/pcre.h
--- otp_src_R13B03.orig/erts/emulator/pcre/pcre.h	2009-03-12 21:16:25.000000000 +0900
+++ otp_src_R13B03.fix-re/erts/emulator/pcre/pcre.h	2009-12-15 14:15:27.859879896 +0900
@@ -311,6 +311,7 @@
 
 #ifdef ERLANG_INTEGRATION
 PCRE_EXP_DECL void  erts_pcre_free_restart_data(void *restart_data);
+PCRE_EXP_DECL void  erts_pcre_restart_data_moved(void *restart_data, const char *, int, int);
 #endif
 #ifdef __cplusplus
 }  /* extern "C" */
diff -urN otp_src_R13B03.orig/erts/emulator/pcre/pcre_exec.c otp_src_R13B03.fix-re/erts/emulator/pcre/pcre_exec.c
--- otp_src_R13B03.orig/erts/emulator/pcre/pcre_exec.c	2009-03-12 21:16:25.000000000 +0900
+++ otp_src_R13B03.fix-re/erts/emulator/pcre/pcre_exec.c	2009-12-15 14:44:52.591599896 +0900
@@ -5389,6 +5389,49 @@
     (erts_pcre_free)(top);
   }
 }
+
+void erts_pcre_restart_data_moved(void* restart_data, const char* old_subject, int slength, int distance)
+{
+  PcreExecContext *exec_context;
+  match_data *md;
+  heapframe *frame; 
+
+#define in_area(ptr,start,nbytes) \
+ ((unsigned long)((char*)(ptr) - (char*)(start)) < (nbytes))
+#define fixup(v,s,e) \
+  if( in_area(v, old_subject+(s), slength+(e)) ){ \
+    v += distance; \
+  }
+
+  exec_context = (PcreExecContext *) restart_data;
+
+  fixup( exec_context->Xstart_match,   0, 1 );
+  fixup( exec_context->Xend_subject,   0, 1 );
+  fixup( exec_context->Xreq_byte_ptr, -1, 0 );
+
+  md = exec_context->Xmd;
+  fixup( md->start_subject,   0, 0 );
+  fixup( md->end_subject,     0, 1 );
+  fixup( md->start_match_ptr, 0, 1 );
+  fixup( md->end_match_ptr,   0, 1 );
+
+  /* first frame is dummy stack. see LOOP_COUNT_RETURN: and LOOP_COUNT_BREAK: */
+  frame = md->state_save;
+  frame = frame != NULL ? frame->Xprevframe : NULL;
+
+  /* actual pcre heapframes. */
+  for(; frame != NULL; frame = frame->Xprevframe )
+  {
+    fixup( frame->Xeptr,   0, 1 );
+    fixup( frame->Xmstart, 0, 1 );
+    fixup( frame->Xpp,         0, 1 );
+    fixup( frame->Xsaved_eptr, 0, 0 );
+  }
+
+#undef in_area
+#undef fixup
+}
+
 #endif
 
 /* End of pcre_exec.c */
